/******************************************************
 * Header file for checkin.c  (version 1)
 ******************************************************
 * 
 *
 * CS328
 * checkin.h
 * 
 * Checkin Group
 *  nparis@cs    835-9638
 *  showsa@nmt   835-6293
 *  rjones@cs    835-2917
 *  tobor@cs     835-6060
 *  jbrackin@nmt 835-1813
 *****************************************************/

/* ====================================================== */
/*                                                        */
/*                  Software Construction                 */
/*                         CS 328                         */
/*                       Fall 1995                        */
/*               Regression Tester Version 2              */
/*                        checkin                         */
/*                          rtrm                          */
/*                         rtlist                         */
/*                                                        */
/*                          team:                         */
/*               Manuel Gallegos -- reality               */
/*                Willard Goosey -- goosey                */
/*               Sean McCullough -- bankshot              */
/*               Jonathan Raloff -- jraloff               */
/*                                                        */
/* ====================================================== */


/*

checkin  --- checkin.c args.c cases.c pcfile.c scan.lex
             file_exist.c fileopt.c howlong.c run_test.c
	     interactive.c

       main
         |
         |--- check_args
         |--- CDIR_exist
         |--- createlist 
         |--- parse_cfile
         |       |
         |       |--- newcase
         |       |--- setcmd 
         |       |--- settno 
         |       |--- setdesc 
         |       |--- settime
         |       |--- setdiff
         |       +--- addoutf
         |
         |--- set_globals
         |       |
         |       +--- output_headers
         |
         |--- interact
         |       |
         |       |--- setcmd 
         |       |--- run_test
         |       |--- setdesc 
         |       |--- settime
         |       |--- setdiff
         |       +--- addoutf
         |
         |--- append_CMDF
         +--- update_CMDF
                 |
                 |--- getcmd 
                 |--- getno 
                 |--- getdesc 
                 |--- gettime
                 |--- getdiff
                 +--- getoutf



rtlist  --- rtlist.c makel.c numlist.c cases.c pcfile.c scan.lex
            file_exist.c fileopt.c args.c

       main
         |
         |--- makel
         |      |
         |      |--- createlist
         |      |--- parse_cfile
         |      |         |
         |      |         |--- newcase
         |      |         |--- setcmd 
         |      |         |--- settno 
         |      |         |--- setdesc 
         |      |         |--- settime
         |      |         |--- setdiff
         |      |         +--- addoutf
         |      |
         |      |--- settag
         |      +--- numlist
         |                |
         |                +--- add_l
         |
         +--- gettag




rtrm    --- rtrm.c makel.c numlist.c cases.c pcfile.c scan.lex
            file_exist.c fileopt.c args.c

       main
         |
         |--- makel
         |      |
         |      |--- createlist
         |      |--- parse_cfile
         |      |         |
         |      |         |--- newcase
         |      |         |--- setcmd 
         |      |         |--- settno 
         |      |         |--- setdesc 
         |      |         |--- settime
         |      |         |--- setdiff
         |      |         +--- addoutf
         |      |
         |      |--- settag
         |      +--- numlist
         |                |
         |                +--- add_l
         |
         |--- output_headers
         |--- append_CMDF
         |--- gettag
         +--- update_CMDF
                 |
                 |--- getcmd 
                 |--- getno 
                 |--- getdesc 
                 |--- gettime
                 |--- getdiff
                 +--- getoutf




          *-- file_exist
          *-- nextcase 
          *-- firstcase 
          *-- lastcase
	  *-- listcases
	  *-- gotolast


                                                   */

/*   checkin.h   */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

int DEBUG;                      /* boolean for debugging */

int yylval;
char readch;
int gtime;
char gdiff[80];

/* *********** defn in lex.yy.c from scan.lex *********** */
extern int yyleng;
extern char yytext[];
extern FILE *yyin;

typedef struct interact {             
        char cmd[80];                   /* all set to NULL on init */
	char ifile[80];
	char dsc[80];
	int tout;
	char dopt[80];
	char ofile[80];
        int cmdline;                    /* all set to zero (0) on init */
	int infile;                     /* and set to one (1) if entered */
	int desc;                       /* as an argument in argv */
	int timeout;
	int diffopt;
	int outfiles;
	int arg;                        /* counter of args */
	char temp[80];                  /* temp storage for strings */
	int gtime;
	int gdiff;
      } INTER;

typedef struct listints {
       int testno;
       struct listints *next;
     } INTLIST;

typedef struct outfile {
       char fname[20];
       struct outfile *nextout;
     } OUTF;

typedef struct casestruct {           /* instance of each case */
       char *cmdln;
       int testno;
       char *desc;
       long int timeout;
       char *diffopt;
       OUTF *output;
       int ERROR;
       int tagged;
     } CASE;

typedef struct teststruct {           /* instance of each test */
       CASE *caseptr;
       struct teststruct *next;
     } TEST;

typedef struct liststruct {           /* defn of class CASES */
       TEST *head;
       TEST *curr;
       TEST *tail;
       int amount;
       } CASES;

CASES *tests;                         /* instance tests of class CASES */


/* ********** filename constructs ********** */
#define   RDIR           "RTresults"           /* location of results dir */
#define   CDIR           "RTtests"             /* location of cases dir */
#define   TMP_OUT        "RTresults/stdout"    /* filename for temp stdout */
#define   TMP_ERR        "RTresults/stderr"    /* filename for temp stderr */
#define   CMDF           "RTtests/commands"    /* filename for cmd file */
#define   FAILED         "RTresults/failed"    /* filename for failed */


/* ********** exit codes ********** */
#define   USAGE         -1              /*  incorrect options usage  */
#define   NOWRITE       -2              /*  unable to write output  */
     /* exit code of 0 indicates success */


/* ********** returned scan codes ********** */
#define   STNO           21
#define   SCMD           22
#define   SDSC           23
#define   STO            24
#define   SDO            25
#define   SOF            26 


/* ==================== args.c ==================== */
INTER check_args ( int argc, char *argv[] );
    /* takes the command line arguments argc, argv
     * parses the command line options for proper config
     * returns INTER class rec (recieved from cmdline)
     * all other options display a usage message and
     * check_args exits with an exit code of USAGE
     */
void debug_msg ( char *msg );
    /* takes in a character string for a message that
     * could be used in debugging (if DEBUG set)
     */
void helpscreen ( char *cmd );
    /* takes a character string (argv[0])
     * and prints out the help screen for available options,
     * meanings, and possible combinations
     * help screens sent to stdout so that it can be saved
     * to a file or piped for 'more'
     */
INTER checkopts ( int argc, char *argv[], INTER srec );
    /* looks at the next argument and fills in the
     * interaction structure as required
     */
INTER getstring ( int argc, char *argv[], INTER rec, int size );
    /* determines if there is a space between the
     * option and the needed text (string/integer)
     */

/* ==================== makel.c ==================== */
CASES *makel ( int argc, char *argv[] );

/* ==================== interactive.c ==================== */
void interactive ( CASES *tests, INTER rec );

/* ==================== run_test.c ==================== */
int run_test ( CASES *tests );

/* ==================== howlong.c ==================== */
int reset_child_time ( void );
int child_time ( void );

/* ==================== pcfile.c ==================== */
int parse_cfile ( void ); 
    /* scans through CMDF getting the desired info
     * setting global values for  gtime, gdiff
     * returns test number that current would be (last + 1)
     */

/* ==================== cases.c ==================== */
void settag (CASES *tests);
    /* sets the tag in the instance tests of class CASES
     */
int gettag (CASES *tests);
    /* returns the tag of the instance tests of class CASES
     * 1 if tagged ... 0 otherwise
     */
int geterror (CASES *tests);
    /* returns the error for the current case
     * takes the instance tests of the class CASES
     */
char *getcmd (CASES *tests);
    /* returns the command line for the current case
     * takes the instance tests of the class CASES
     */
int getno (CASES *tests);
    /* returns the test number for the current case
     * takes the instance tests of the class CASES
     */
char *getdesc (CASES *tests);
    /* returns the description for the current case
     * takes the instance tests of the class CASES
     */
int gettime (CASES *tests);
    /* returns the time limit for the current case
     * takes the instance tests of the class CASES
     */
char *getdiff (CASES *tests);
    /* returns the diff options for the current case
     * takes the instance tests of the class CASES
     */
char *getoutf (CASES *tests);
    /* returns the output files for the current case (as 1 string)
     * takes the instance tests of the class CASES
     */
void nextcase (CASES *tests);
    /* takes the instance tests of the class CASES
     * moves to the next case 
     */
void firstcase (CASES *tests);
    /* takes the instance tests of the class CASES
     * moves to the first case listed in the instance
     */
int lastcase (CASES *tests);
    /* returns 1 if at last instance of CASES, 0 otherwise
     * takes the instances tests of the class CASES
     */
CASES *createlist ( );
    /* returns an instance of CASES
     * creates an instance of CASES 
     */
void newcase (CASES *tests, int newtno);
    /* takes the instance tests of the class CASES
     * creates a new case for the object tests
     */
void setcmd (CASES *tests, char *cmd);
    /* takes the instance tests of the class CASES and the command line
     * places cmd into the current instance of CASES
     */
void settno (CASES *tests, int tno);
    /* takes the instance tests of the class CASES and the test number
     * places tno into the current instance of CASES
     */
void setdesc (CASES *tests, char *desc);
    /* takes the instance tests of the class CASES and the description
     * places desc into the current instance of CASES
     */
void settime (CASES *tests, int time);
    /* takes the instance tests of the class CASES and the time limit
     * places time into the current instance of CASES
     */
void setdiff (CASES *tests, char *dopt);
    /* takes the instance tests of the class CASES and the diff options
     * places dopt into the current instance of CASES
     */
void setoutf (CASES *tests, char *outfiles);
    /* takes the instance tests of the class CASES and outfiles
     * places outfiles into the current instance of CASES through OUTF
     */
void addoutf (CASES *tests, char *outfile);
    /* takes the instance tests of the class CASES and the output file
     * adds outfile to list of OUTF in the current instance of CASES
     */
void seterror (CASES *tests, int error);
    /* takes the instance tests of the class CASES and the error
     * places error into the current instance of CASES
     */
void listcases (CASES *tests);
    /* takes the class CASES and lists every instance
     */
void gotolast (CASES *tests);
    /* places the current pointer at the last instance of
     * the class CASES
     */

/* ==================== fileopt.c ==================== */
int CDIR_made ( void );
    /* checks if CDIR exists and creates it if it does not
     * and writes out global headers to CMDF if CMDF just created
     * returns 1 if CDIR just created, 0 otherwise
     */
int output_header ( void );
    /* writes out global headers to CMDF
     * returns 1 if headers just writen, 0 otherwise
     */
void update_CMDF ( CASES *tests, FILE *cmdptr );
    /* adds the current instance of CASES to CMDF
     */
FILE *append_CMDF ( void );
    /* opens CMDF for appending
     * returns the file stream pointer
     */

/* ==================== file_exist.c ==================== */
int file_exist (char *file_name);
    /* returns 0,1,2,3 
     * takes a string file_name
     * checks if file_name exists 
     * returns:
     *         1 --  executable file
     *         2 --  readable file
     *         3 --  readable/writable directory
     */

