/*****************************************************************************
   rt v1
 * Willard Goosey
 * exec_cmd.c
 * part of RT
 * cs328 fall 95
 ****************************************************************************

  rt v2, 20 November 1995, Randall Jones (rjones@cs.nmt.edu)

  Notes:
   
 ****************************************************************************/
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/wait.h>
#include <signal.h>
#include <sys/time.h>      /* setrlimit */
#include <sys/resource.h>      /* setrlimit */
#include <unistd.h>      /* setrlimit */

#include "rt.h"

#define  SHMEM_KEY   ((key_t)(47))
#define  SHMEM_SIZE  sizeof(int)

extern int GlobalTimeout;

int intLength(int testInt);

int exec_cmd (CASES *tests)
{
  int id;
  int timeout;
  int testno;
  int err;
  pid_t cPid;
  char *testStdin, *testStdout, *testStderr, *testExit;

#ifdef DBUGMSG
  /*found*/debug_msg (" exec_cmd: <begin>");
#endif

  testno = getno(tests);

  /* get the timeout for this test case */
  /**** could be (*should be*) a gettimout function in cases.c */
  timeout = tests->curr->caseptr->timeout;
  if (timeout < 0)		/* no local (test case) timeout defined */
  {
    timeout = GlobalTimeout;	/* minutes */
  }

  testStdin = (char *) malloc (sizeof(char) * (strlen(CDIR) +
					       strlen(TEST_LBL) +
					       intLength(testno) +
					       strlen(INPUT_EXT) + 1));
  sprintf(testStdin, "%s/%s%d%s", CDIR, TEST_LBL, testno, INPUT_EXT);

  testStdout = (char *) malloc (sizeof(char) * (strlen(RDIR) +
						strlen(TEST_LBL) +
						intLength(testno) +
						strlen(STDOUT_EXT) + 1));
  sprintf(testStdout, "%s/%s%d%s", RDIR, TEST_LBL, testno, STDOUT_EXT);

  testStderr = (char *) malloc (sizeof(char) * (strlen(RDIR) +
						strlen(TEST_LBL) +
						intLength(testno) +
						strlen(STDERR_EXT) + 1));
  sprintf(testStderr, "%s/%s%d%s", RDIR, TEST_LBL, testno, STDERR_EXT);

  testExit = (char *) malloc (sizeof(char) * (strlen(RDIR) +
					      strlen(TEST_LBL) +
					      intLength(testno) +
					      strlen(EXITCODE_EXT) + 1));
  sprintf(testExit, "%s/%s%d%s", RDIR, TEST_LBL, testno, EXITCODE_EXT);

#ifdef DBUGMSG
  sprintf(dbgMsg," exec_cmd: cl=%s", getcmd(tests));
  /*found*/debug_msg (dbgMsg);
#endif

  if ((cPid=fork()) > 0)	/*parent*/
  {
    /* wait for child to finish ( killed or clean ) */
    wait (&err);
    
    /* check err */
#ifdef DBUGMSG
    if (err & 0x00FF)  /* if low order 8 bits of err are non-zero */
    {
      sprintf (dbgMsg, " exec_cmd: test case process %d timed out "
	       "(%d CPU min.), killed\n", cPid, timeout);
      /*found*/debug_msg(dbgMsg);
    }
#endif
    /* else: test case had a clean (on time) finish */

  }
  else if (cPid == 0)		/*child*/
  {  
    struct rlimit rlimitStruct;	/* resource limit struct */
    struct rlimit *rlPtr;
    FILE *exitcodeFp;
    int exitCode;

    freopen (testStdin, "r", stdin);
    freopen (testStdout, "w+", stdout);
    freopen (testStderr, "w+", stderr);
    exitcodeFp = fopen(testExit, "w+");

    /* set resource limits - set Maximum CPU usage (if a timeout exists) */
    if (timeout > 0)
    {
      rlPtr = &rlimitStruct;
      rlPtr->rlim_cur = timeout * 60; /* limit in CPU seconds */
      rlPtr->rlim_max = timeout * 60;
      if (setrlimit(RLIMIT_CPU, rlPtr))
      {
	/* freopen (stderr, "w+", testStderr); */
	fprintf(stderr,"error: setting resource (CPU) limits for test case\n");
	exit(-1);
      }
    }
    
    exitCode = system (getcmd(tests));
    exitCode = exitCode >> 8;/*0xXX00 is returned from system XX = exit code*/

    fprintf(exitcodeFp, "%d\n", exitCode);
    fflush (stdout);
    fflush (stderr);
    fclose (stdin);
    fclose (stdout);
    fclose (stderr);
    fclose (exitcodeFp);

    exit (exitCode);
  }

  /*sh returns exit code of last command run
   *system returns exit code of sh */
  /* from man page of system:
   *   its termination status is contained in
   *   the 8 bits higher up from the low-order 8 bits of the  value
   *   returned by wait().
   */
  err = err >> 8;

#ifdef DBUGMSG
  sprintf (dbgMsg, " exec_cmd: wait returned %03x pid was %03x", err, cPid);
  /*found*/debug_msg (dbgMsg);

  /*found*/debug_msg (" exec_cmd: <end>");
#endif

  return (err);
}
      

int intLength(int testInt)
{
  int count = 0;
  float f;

  f = (float) testInt;

  while(f > 0.0)
  {
    f /= 10.0;
    count++;
  }
  return count;
}

