/*****************************************************************************
    FILE       : rtresults.c

    PURPOSE    : Display Specific failed results from regression tester.

    RESP PARTY : Bill Crum 11/15/95
               : 11/28/95 Changes from Software Inspection

    NOTES      : Program will open "./RTresults/results" if it exists and
                 display the errors from each test specified in the range.
                 Range is in the form of nnn-nnn, nnn-nnn, nnn, nnn.
		 Commas are not required if spaces are used instead.
		 Duplicate numbers are ok, but the test will be displayed
		 only once. No range selected means to display all failed
		 tests. 
 *****************************************************************************/



/* ^L*/
/*****************************************************************************
    INCLUDE FILES
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>



/* ^L*/
/*****************************************************************************
    GLOBAL VARIABLES
 *****************************************************************************/

struct range {              /* linked list structure */
  int testNumber;
  struct range *next;
};

struct range rangeTop;     /* Just the first one and it is empty */

int all;                   /* Set if display all bad results */

/* ^L*/
/*****************************************************************************
    FUNCTION     : main

    PURPOSE      : Display Specific failed results from regression tester.

    RETURN VALUE : None

    NOTES        : main uses the following two routines:
	                     int inTheRange(testnumber)
	                     int load_range(argc,argv)
 *****************************************************************************/

main(argc, argv)
int argc;
char *argv[];
{
  int testNumber;              /* Test number derived from results file */
  char buff[512];         /* Input buffer for strings from results file */
  char testStr[10];       /* Used to parse line with test number on it  */
  struct range *rangePtr;         /* pointer into linked list of ranges */
  FILE *fp;
  
  all = load_range(argc,argv);       /* all = 1 print out all of results*/
  /* all = 0 print out range */
/*  print_range();*/
  if ((fp = fopen("./RTresults/results","r")) == NULL)
    {
      fprintf(stdout,"There are no results to print.\n");
      exit(0);
    }
  
  fgets(buff,512,fp);
  while (1)
    {
      sscanf(buff, "%s %d",testStr,&testNumber);
      if (strcmp(testStr,"::test") == 0)
	{
	  if(inTheRange(testNumber))
	    {
	      fprintf(stdout,
		      "\n\n**********************************\n");
	      fprintf(stdout,
		      "This is the results for test #%d\n",testNumber);
	      fprintf(stdout,
		      "**********************************\n\n");
	      while (1)
		{
		  if(fgets(buff,512,fp) == NULL)  /* end of file */
		    {
		      exit(0);
		    }
		  else
		    {
		      if(buff[0] == '\n') /* Line with cr in one spot only */
			{
			  fprintf(stdout,"\n\n");
			  exit(0);
			}
		      /* Look for next test number line */
		      if(buff[0] != ':')
			fprintf(stdout,"%s",buff);
		      else
			break;
		    }
		}
	    }/* if(inTheRange(testNumber)) */
	  /* If not in the range, still have to read in to find next test */
	  else
	    {
	      while (1)
		{
		  if(fgets(buff,512,fp) == NULL)   /* end of file */
		    {
		      fprintf(stdout,"\n\n");
		      exit(0);
		    }
		  else
		    {
		      if(buff[0] == ':')          /* end of test */
			break;
		      if(buff[0] == '\n')         /* end of file */
			{
			  fprintf(stdout,"\n\n");
			  exit(0);
			}
		    }
		}
	    }
	}/* if (strcmp(testStr,"::test") == 0) */
    } /* while (1) */
  
}

/* ^L*/
/*****************************************************************************
    FUNCTION     : inTheRange

    PURPOSE      : Checks if test number from results file is one to be
                   displayed. If global variable all is set then all tests
                   will be displayed.

    RETURN VALUE : returns 1 if in range, 0 if not.

    NOTES        :
 *****************************************************************************/


int inTheRange(testnumber)
int testnumber;
{
  struct range *rangePtr;
  
  rangePtr = &rangeTop; /* rangeTop is empty, points to first record */
  if (all)
    return(1);
  while(rangePtr->next != NULL)
    {
      rangePtr = rangePtr->next;
      if (testnumber == rangePtr->testNumber)
	{
	  return(1);  /* Found it, go ahead and display it */
	}
    }
  return(0);
}

/* ^L*/
/*****************************************************************************
    FUNCTION     : loadRange

    PURPOSE      : Takes the command line parameters and creates a linked list
                       of numbers to be used to display those tests.

    RETURN VALUE : Returns a 1 if all are to be displayed, 0 if range is built.
                   Exits if usage is improper.

    NOTES        : Will correct reversed ranges. example 23-11
 *****************************************************************************/

int load_range(argc,argv)
int argc;
char *argv[];
{
  int count;   /* Keep track of command line arguments */
  int offset;  /* Offset into each argument */
  struct range *rangePtr;   /* pointer into list */
  int number;               /* number made from command line argument */
  int spread;        /* spread is starting number of range spread-nnn */
  int upper;         /* upper is ending number of range nnn-upper */
  char strbyte;      /* For looking at a character in the arrays */
  char buff[80];     /* temp storage for command line arrays */
  
  if (argc > 1)  /* find a range */
    {
      count = 1;
      rangePtr = &rangeTop;
      while (count < argc)
	{
	  offset = 0;
	  spread = -1;
	  strcpy(buff,argv[count]);
	  strbyte = buff[offset];
	  number = -1;            /* Dont want a list of 0's */
	  
	  while(1)
	    {
	      if (isdigit(strbyte)) /* if its a number build it up */
		{                     /* left to right */
		  if (number < 0)  /* To make sure got a number before ','*/
		    number = 0;
		  number *= 10;
		  number += (strbyte -'0');
		  offset++;
		  strbyte = buff[offset];
		}
	      else
		{
		  if (number < 0) /* Might of got a ',' without number */
		    {
		      fprintf(stdout,"Usage rtresults [nnn-nnn,nnn]\n");
		      exit(0);
		    }
		  rangePtr->next =
		    (struct range *) calloc(1, sizeof(struct range));
		  if (rangePtr->next == NULL)
		    {
		      fprintf(stdout,"1Error with calloc\n");
		      exit(1);
		    }
		  if (spread >= 0)
		    {
		      if (spread > number)      /* Fix reverse numbers */
			{
			  rangePtr->testNumber = number;
			  number = spread;
			  spread = rangePtr->testNumber;
			}
		      while (spread >= 0)       /* Fill in between numbers */
			{
			  spread++;
			  upper = number;
			  rangePtr->next =
			    (struct range *) calloc(1, sizeof(struct range));
			  if (rangePtr->next == NULL)
			    {
			      fprintf(stdout,"2Error with calloc\n");
			      exit(1);
			    }
			  rangePtr = rangePtr->next;
			  rangePtr->testNumber = spread;
			  rangePtr->next = (struct range *) NULL;
			  if (spread == upper)
			    spread = -2;/* already did range */
			}
		    }
		  else
		    {
		      rangePtr = rangePtr->next;
		      rangePtr->testNumber = number;
		      rangePtr->next = (struct range *) NULL;
		      spread = -1;
		    }
		  number = -1;
		  if (strbyte == '\0')
		    break;
		  
		  /* Use spread as a flag and a starting number of range */
		  
		  if (strbyte == '-')
		    if(spread == -2)
		    {
		      fprintf(stdout,"Usage rtresults [nnn-nnn,nnn]\n");
		      exit(0);
		    }
		    else
		      spread = rangePtr->testNumber; 
		  		  
		  offset++;
		  strbyte = buff[offset]; /* check next char of arg    */
		  if (strbyte == '\0')    /* get next command line arg */
		    break;
		}/* else */
	    }/* forever */
	  count++;
	} /* while (count < argc) */
      return(0);
    } /* if (argc > 1) */
  return(1);
}

/* print_range is for debuggin only */

print_range()
{
  struct range * rangePtr;
  if (all)
    {
      return;
    }
  
  rangePtr = &rangeTop;
  for (;;)
    {
      if (rangePtr->next == NULL)
	return;
      rangePtr = rangePtr->next;
      printf("%d\n",rangePtr->testNumber);
    }
  
}


